# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import platform
from urllib.request import urlopen

from knack.util import CLIError
from knack.log import get_logger

from azure.cli.core.util import in_cloud_console, user_confirmation

from ._docker_utils import (
    get_access_credentials,
    request_data_from_registry,
    RegistryException,
    HelmAccessTokenPermission
)


logger = get_logger(__name__)


def acr_helm_list(cmd,
                  registry_name,
                  repository='repo',
                  resource_group_name=None,  # pylint: disable=unused-argument
                  tenant_suffix=None,
                  username=None,
                  password=None):
    login_server, username, password = get_access_credentials(
        cmd=cmd,
        registry_name=registry_name,
        tenant_suffix=tenant_suffix,
        username=username,
        password=password,
        artifact_repository=repository,
        permission=HelmAccessTokenPermission.PULL.value)

    return request_data_from_registry(
        http_method='get',
        login_server=login_server,
        path=_get_charts_path(repository),
        username=username,
        password=password)[0]


def acr_helm_show(cmd,
                  registry_name,
                  chart,
                  version=None,
                  repository='repo',
                  resource_group_name=None,  # pylint: disable=unused-argument
                  tenant_suffix=None,
                  username=None,
                  password=None):
    login_server, username, password = get_access_credentials(
        cmd=cmd,
        registry_name=registry_name,
        tenant_suffix=tenant_suffix,
        username=username,
        password=password,
        artifact_repository=repository,
        permission=HelmAccessTokenPermission.PULL.value)

    return request_data_from_registry(
        http_method='get',
        login_server=login_server,
        path=_get_charts_path(repository, chart, version),
        username=username,
        password=password)[0]


def acr_helm_delete(cmd,
                    registry_name,
                    chart,
                    version=None,
                    repository='repo',
                    resource_group_name=None,  # pylint: disable=unused-argument
                    tenant_suffix=None,
                    username=None,
                    password=None,
                    prov=False,
                    yes=False):
    if version:
        message = "This operation will delete the chart package '{}'".format(
            _get_chart_package_name(chart, version, prov))
    else:
        message = "This operation will delete all versions of the chart '{}'".format(chart)
    user_confirmation("{}.\nAre you sure you want to continue?".format(message), yes)

    login_server, username, password = get_access_credentials(
        cmd=cmd,
        registry_name=registry_name,
        tenant_suffix=tenant_suffix,
        username=username,
        password=password,
        artifact_repository=repository,
        permission=HelmAccessTokenPermission.DELETE.value)

    return request_data_from_registry(
        http_method='delete',
        login_server=login_server,
        path=_get_blobs_path(repository, chart, version, prov) if version else _get_charts_path(repository, chart),
        username=username,
        password=password)[0]


def acr_helm_push(cmd,
                  registry_name,
                  chart_package,
                  repository='repo',
                  force=False,
                  resource_group_name=None,  # pylint: disable=unused-argument
                  tenant_suffix=None,
                  username=None,
                  password=None):
    from os.path import isdir, basename

    if isdir(chart_package):
        raise CLIError("Please run 'helm package {}' to generate a chart package first.".format(chart_package))

    login_server, username, password = get_access_credentials(
        cmd=cmd,
        registry_name=registry_name,
        tenant_suffix=tenant_suffix,
        username=username,
        password=password,
        artifact_repository=repository,
        permission=HelmAccessTokenPermission.PUSH_PULL.value)

    path = _get_blobs_path(repository, basename(chart_package))

    try:
        result = request_data_from_registry(
            http_method='patch' if force else 'put',
            login_server=login_server,
            path=path,
            username=username,
            password=password,
            file_payload=chart_package)[0]
        return result
    except RegistryException as e:
        # Fallback using PUT if the chart doesn't exist
        if e.status_code == 404 and force:
            return request_data_from_registry(
                http_method='put',
                login_server=login_server,
                path=path,
                username=username,
                password=password,
                file_payload=chart_package)[0]
        raise


def acr_helm_repo_add(cmd,
                      registry_name,
                      repository='repo',
                      resource_group_name=None,  # pylint: disable=unused-argument
                      tenant_suffix=None,
                      username=None,
                      password=None):
    helm_command, _ = get_helm_command()

    login_server, username, password = get_access_credentials(
        cmd=cmd,
        registry_name=registry_name,
        tenant_suffix=tenant_suffix,
        username=username,
        password=password,
        artifact_repository=repository,
        permission=HelmAccessTokenPermission.PULL.value)

    from subprocess import Popen
    p = Popen([helm_command, 'repo', 'add', registry_name,
               'https://{}/helm/v1/{}'.format(login_server, repository),
               '--username', username, '--password', password])
    p.wait()


def acr_helm_install_cli(client_version='2.16.3', install_location=None, yes=False):
    """Install Helm command-line tool."""

    if client_version >= '3':
        logger.warning('Please note that "az acr helm" commands do not work with Helm 3, '
                       'but you can still push Helm chart to ACR using a different command flow. '
                       'For more information, please check out '
                       'https://docs.microsoft.com/azure/container-registry/container-registry-helm-repos')

    install_location, install_dir, cli = _process_helm_install_location_info(install_location)

    client_version = "v%s" % client_version
    source_url = 'https://get.helm.sh/{}'
    package, folder = _get_helm_package_name(client_version)
    download_path = ''

    if not package:
        raise CLIError('No prebuilt binary for current system.')

    try:
        import tempfile
        with tempfile.TemporaryDirectory() as tmp_dir:
            download_path = os.path.join(tmp_dir, package)
            _urlretrieve(source_url.format(package), download_path)
            _unzip(download_path, tmp_dir)

            sub_dir = os.path.join(tmp_dir, folder)
            # Ask user to check license
            if not yes:
                with open(os.path.join(sub_dir, 'LICENSE')) as f:
                    text = f.read()
                    logger.warning(text)
                user_confirmation('Before proceeding with the installation, '
                                  'please confirm that you have read and agreed the above license.')

            # Move files from temporary location to specified location
            import shutil
            import stat
            for f in os.scandir(sub_dir):
                # Rename helm to specified name
                target_path = install_location if os.path.splitext(f.name)[0] == 'helm' \
                    else os.path.join(install_dir, f.name)
                logger.debug('Moving %s to %s', f.path, target_path)
                shutil.move(f.path, target_path)

                if os.path.splitext(f.name)[0] in ('helm', 'tiller'):
                    os.chmod(target_path, os.stat(target_path).st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
    except IOError as e:
        import traceback
        logger.debug(traceback.format_exc())
        raise CLIError('Error while installing {} to {}: {}'.format(cli, install_dir, e))

    logger.warning('Successfully installed %s to %s.', cli, install_dir)
    # Remind user to add to path
    system = platform.system()
    if system == 'Windows':  # be verbose, as the install_location likely not in Windows's search PATHs
        env_paths = os.environ['PATH'].split(';')
        found = next((x for x in env_paths if x.lower().rstrip('\\') == install_dir.lower()), None)
        if not found:
            # pylint: disable=logging-format-interpolation
            logger.warning('Please add "{0}" to your search PATH so the `{1}` can be found. 2 options: \n'
                           '    1. Run "set PATH=%PATH%;{0}" or "$env:path += \'{0}\'" for PowerShell. '
                           'This is good for the current command session.\n'
                           '    2. Update system PATH environment variable by following '
                           '"Control Panel->System->Advanced->Environment Variables", and re-open the command window. '
                           'You only need to do it once'.format(install_dir, cli))
    else:
        logger.warning('Please ensure that %s is in your search PATH, so the `%s` command can be found.',
                       install_dir, cli)


def get_helm_command(is_diagnostics_context=False):
    from ._errors import HELM_COMMAND_ERROR
    helm_command = 'helm'

    from subprocess import PIPE, Popen
    try:
        p = Popen([helm_command, "--help"], stdout=PIPE, stderr=PIPE)
        _, stderr = p.communicate()
    except OSError as e:
        logger.debug("Could not run '%s' command. Exception: %s", helm_command, str(e))
        # The executable may not be discoverable in WSL so retry *.exe once
        try:
            helm_command = 'helm.exe'
            p = Popen([helm_command, "--help"], stdout=PIPE, stderr=PIPE)
            _, stderr = p.communicate()
        except OSError as inner:
            logger.debug("Could not run '%s' command. Exception: %s", helm_command, str(inner))
            if is_diagnostics_context:
                return None, HELM_COMMAND_ERROR
            raise CLIError(HELM_COMMAND_ERROR.get_error_message())

    if stderr:
        if is_diagnostics_context:
            return None, HELM_COMMAND_ERROR.append_error_message(stderr.decode())
        raise CLIError(HELM_COMMAND_ERROR.append_error_message(stderr.decode()).get_error_message())

    return helm_command, None


def _get_charts_path(repository, chart=None, version=None):
    if chart and version:
        return '/helm/v1/{}/_charts/{}/{}'.format(repository, chart, version)

    if chart:
        return '/helm/v1/{}/_charts/{}'.format(repository, chart)

    return '/helm/v1/{}/_charts'.format(repository)


def _get_blobs_path(repository, chart, version=None, prov=False):
    path = '/helm/v1/{}/_blobs'.format(repository)

    if version:
        return '{}/{}'.format(path, _get_chart_package_name(chart, version, prov))

    return '{}/{}'.format(path, chart)


def _get_chart_package_name(chart, version, prov=False):
    chart_package_name = '{}-{}.tgz'.format(chart, version)

    if prov:
        return '{}.prov'.format(chart_package_name)

    return chart_package_name


def _process_helm_install_location_info(install_location):
    if not install_location or install_location.isspace():
        raise CLIError('Invalid install location.')

    install_dir, cli = os.path.dirname(install_location), os.path.basename(install_location)
    if not install_dir:
        # Use current working directory
        install_dir = os.getcwd()
        install_location = os.path.join(install_dir, cli)
    # Ensure installation directory exists
    if not os.path.exists(install_dir):
        os.makedirs(install_dir)
    if not cli:
        system = platform.system()
        cli = 'helm.exe' if system == 'Windows' else 'helm'
        install_location = os.path.join(install_dir, cli)

    return install_location, install_dir, cli


def _get_helm_package_name(client_version):
    package_template = 'helm-{}-{}-{}.{}'
    folder_template = '{}-{}'
    package = ''
    folder = ''

    # Reference: https://github.com/helm/helm/blob/master/scripts/get
    archs = {
        'armv5': 'armv5',
        'armv6': 'armv6',
        'armv7': 'arm',
        'aarch64': 'arm64',
        'x86': '386',
        'x86_64': 'amd64',
        'i686': '386',
        'i386': '386',
        'AMD64': 'amd64',
        'ppc64le': 'ppc64le',
        's390x': 's390x'
    }
    machine = platform.machine()
    if machine not in archs:
        return None, None
    arch = archs[machine]

    system = platform.system().lower()
    if system == 'windows':
        package = package_template.format(client_version, system, arch, 'zip')
    elif system in ('linux', 'darwin'):
        package = package_template.format(client_version, system, arch, 'tar.gz')
    else:
        return None, None

    folder = folder_template.format(system, arch)
    return package, folder


def _ssl_context():
    import sys
    import ssl

    if sys.version_info < (3, 4) or (in_cloud_console() and platform.system() == 'Windows'):
        try:
            return ssl.SSLContext(ssl.PROTOCOL_TLS)  # added in python 2.7.13 and 3.6
        except AttributeError:
            return ssl.SSLContext(ssl.PROTOCOL_TLSv1)

    return ssl.create_default_context()


def _urlretrieve(url, path):
    logger.warning('Downloading client from %s, it may take a long time...', url)
    with urlopen(url, context=_ssl_context()) as response:
        logger.debug('Start downloading from %s to %s', url, path)
        # Open for writing in binary mode
        with open(path, "wb") as f:
            f.write(response.read())
    logger.debug('Successfully downloaded from %s to %s', url, path)


def _unzip(src, dest):
    logger.debug('Extracting %s to %s.', src, dest)
    system = platform.system()
    if system == 'Windows':
        import zipfile
        with zipfile.ZipFile(src, 'r') as zipObj:
            zipObj.extractall(dest)
    elif system in ('Linux', 'Darwin'):
        import tarfile
        with tarfile.open(src, 'r') as tarObj:
            tarObj.extractall(dest)
    else:
        raise CLIError('The current system is not supported.')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def process_condition_parameter(namespace):
    from azure.mgmt.monitor.models import ActivityLogAlertAllOfCondition
    from knack.util import CLIError

    try:
        expression = namespace.condition
        if expression is None:
            return
    except AttributeError:
        return

    error = 'incorrect usage: --condition requires an expression in the form of FIELD=VALUE[ and FIELD=VALUE...]'

    if not expression:
        raise CLIError(error)

    if len(expression) == 1:
        expression = [each.strip() for each in expression[0].split(' ')]
    elif isinstance(expression, list):
        expression = [each.strip() for each in expression]
    else:
        raise CLIError(error)

    if not expression or not len(expression) % 2:
        raise CLIError(error)

    # This is a temporary approach built on the assumption that there is only AND operators. Eventually, a proper
    # YACC will be created to parse complex condition expression.

    # Ensure all the string at even options are AND operator
    operators = [expression[i] for i in range(1, len(expression), 2)]
    if any(op != 'and' for op in operators):
        raise CLIError(error)

    # Pick the strings at odd position and convert them into condition leaf.
    conditions = dict(_normalize_condition(expression[i]) for i in range(0, len(expression), 2))
    setattr(namespace, 'condition', ActivityLogAlertAllOfCondition(all_of=list(conditions.values())))


def list_activity_logs_alert(client, resource_group_name=None):
    if resource_group_name:
        return client.list_by_resource_group(resource_group_name)

    return client.list_by_subscription_id()


def create(cmd, client, resource_group_name, activity_log_alert_name, scopes=None, condition=None,
           action_groups=frozenset(), tags=None, disable=False, description=None, webhook_properties=None):
    from msrestazure.tools import resource_id
    from azure.mgmt.monitor.models import (ActivityLogAlertResource, ActivityLogAlertAllOfCondition,
                                           ActivityLogAlertLeafCondition, ActivityLogAlertActionList)
    from azure.mgmt.monitor.models import ActivityLogAlertActionGroup as ActionGroup
    from azure.cli.core.commands.client_factory import get_subscription_id
    from knack.util import CLIError

    if not scopes:
        scopes = [resource_id(subscription=get_subscription_id(cmd.cli_ctx), resource_group=resource_group_name)]

    if _get_alert_settings(client, resource_group_name, activity_log_alert_name, throw_if_missing=False):
        raise CLIError('The activity log alert {} already exists in resource group {}.'.format(activity_log_alert_name,
                                                                                               resource_group_name))

    # Add alert conditions
    condition = condition or ActivityLogAlertAllOfCondition(
        all_of=[ActivityLogAlertLeafCondition(field='category', equals='ServiceHealth')])

    # Add action groups
    action_group_rids = _normalize_names(cmd.cli_ctx, action_groups, resource_group_name, 'microsoft.insights',
                                         'actionGroups')
    action_groups = [ActionGroup(action_group_id=i, webhook_properties=webhook_properties) for i in action_group_rids]
    alert_actions = ActivityLogAlertActionList(action_groups=action_groups)

    settings = ActivityLogAlertResource(location='global', scopes=scopes, condition=condition,
                                        actions=alert_actions, enabled=not disable, description=description, tags=tags)

    return client.create_or_update(resource_group_name=resource_group_name,
                                   activity_log_alert_name=activity_log_alert_name, activity_log_alert=settings)


def add_scope(client, resource_group_name, activity_log_alert_name, scopes, reset=False):
    settings = _get_alert_settings(client, resource_group_name, activity_log_alert_name)

    new_scopes = set() if reset else set(settings.scopes)
    for scope in scopes:
        new_scopes.add(scope)

    settings.scopes = list(new_scopes)
    return client.create_or_update(resource_group_name=resource_group_name,
                                   activity_log_alert_name=activity_log_alert_name,
                                   activity_log_alert=settings)


def remove_scope(client, resource_group_name, activity_log_alert_name, scopes):
    settings = _get_alert_settings(client, resource_group_name, activity_log_alert_name)

    new_scopes = set(settings.scopes)
    for scope in scopes:
        try:
            new_scopes.remove(scope)
        except KeyError:
            pass

    settings.scopes = list(new_scopes)
    return client.create_or_update(resource_group_name=resource_group_name,
                                   activity_log_alert_name=activity_log_alert_name,
                                   activity_log_alert=settings)


def add_action_group(cmd, client, resource_group_name, activity_log_alert_name, action_group_ids, reset=False,
                     webhook_properties=None, strict=False):
    from azure.mgmt.monitor.models import ActivityLogAlertActionGroup as ActionGroup

    settings = _get_alert_settings(client, resource_group_name, activity_log_alert_name)

    # normalize the action group ids
    rids = _normalize_names(cmd.cli_ctx, action_group_ids, resource_group_name, 'microsoft.insights', 'actionGroups')

    if reset:
        action_groups = [ActionGroup(action_group_id=rid, webhook_properties=webhook_properties) for rid in rids]
    else:
        action_groups = dict((each.action_group_id, each) for each in settings.actions.action_groups)

        for rid in rids:
            if rid in action_groups and webhook_properties != action_groups[rid].webhook_properties and strict:
                raise ValueError('Fails to override webhook properties of action group {} in strict mode.'.format(rid))

            action_groups[rid] = ActionGroup(action_group_id=rid, webhook_properties=webhook_properties)
        action_groups = list(action_groups.values())

    settings.actions.action_groups = action_groups
    return client.create_or_update(resource_group_name=resource_group_name,
                                   activity_log_alert_name=activity_log_alert_name,
                                   activity_log_alert=settings)


def remove_action_group(cmd, client, resource_group_name, activity_log_alert_name, action_group_ids):
    settings = _get_alert_settings(client, resource_group_name, activity_log_alert_name)

    if len(action_group_ids) == 1 and action_group_ids[0] == '*':
        settings.actions.action_groups = []
    else:
        # normalize the action group ids
        rids = _normalize_names(cmd.cli_ctx, action_group_ids, resource_group_name, 'microsoft.insights',
                                'actionGroups')
        action_groups = [each for each in settings.actions.action_groups if each.action_group_id not in rids]
        settings.actions.action_groups = action_groups

    return client.create_or_update(resource_group_name=resource_group_name,
                                   activity_log_alert_name=activity_log_alert_name,
                                   activity_log_alert=settings)


def update(instance, condition=None, enabled=None, tags=None, description=None):
    if tags:
        instance.tags = tags

    if description:
        instance.description = description

    if enabled is not None:
        instance.enabled = enabled

    if condition is not None:
        instance.condition = condition

    return instance


# pylint: disable=inconsistent-return-statements
def _normalize_condition(condition_instance):
    from azure.mgmt.monitor.models import ActivityLogAlertLeafCondition

    if isinstance(condition_instance, str):
        try:
            field, value = condition_instance.split('=')
            return '{}={}'.format(field.lower(), value), ActivityLogAlertLeafCondition(field=field, equals=value)
        except ValueError:
            # too many values to unpack or not enough values to unpack
            raise ValueError('Condition "{}" does not follow format FIELD=VALUE'.format(condition_instance))
    elif isinstance(condition_instance, ActivityLogAlertLeafCondition):
        return '{}={}'.format(condition_instance.field.lower(), condition_instance.equals), condition_instance


def _normalize_names(cli_ctx, resource_names, resource_group, namespace, resource_type):
    """Normalize a group of resource names. Returns a set of resource ids. Throws if any of the name can't be correctly
    converted to a resource id."""
    from msrestazure.tools import is_valid_resource_id, resource_id
    from azure.cli.core.commands.client_factory import get_subscription_id

    rids = set()
    # normalize the action group ids
    for name in resource_names:
        if is_valid_resource_id(name):
            rids.add(name)
        else:
            rid = resource_id(subscription=get_subscription_id(cli_ctx),
                              resource_group=resource_group,
                              namespace=namespace,
                              type=resource_type,
                              name=name)
            if not is_valid_resource_id(rid):
                raise ValueError('The resource name {} is not valid.'.format(name))
            rids.add(rid)

    return rids


def _get_alert_settings(client, resource_group_name, activity_log_alert_name, throw_if_missing=True):
    from azure.core.exceptions import HttpResponseError

    try:
        return client.get(resource_group_name=resource_group_name, activity_log_alert_name=activity_log_alert_name)
    except HttpResponseError as ex:
        from knack.util import CLIError
        if ex.status_code == 404:
            if throw_if_missing:
                raise CLIError('Can\'t find activity log alert {} in resource group {}.'.format(activity_log_alert_name,
                                                                                                resource_group_name))
            return None
        raise CLIError(ex.message)



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from codecs import open
from setuptools import setup, find_packages
import sys

try:
    from azure_cli_bdist_wheel import cmdclass
except ImportError:
    from distutils import log as logger

    logger.warn("Wheel is not available, disabling bdist_wheel hook")
    cmdclass = {}

VERSION = "2.30.0.post20211115155801"
# If we have source, validate that our version numbers match
# This should prevent uploading releases with mismatched versions.
try:
    with open('azure/cli/__main__.py', 'r', encoding='utf-8') as f:
        content = f.read()
except OSError:
    pass
else:
    import re

    m = re.search(r'__version__\s*=\s*[\'"](.+?)[\'"]', content)
    if not m:
        print('Could not find __version__ in azure/cli/__main__.py')
        sys.exit(1)
    if m.group(1) != VERSION:
        print('Expected __version__ = "{}"; found "{}"'.format(VERSION, m.group(1)))
        sys.exit(1)

CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Intended Audience :: System Administrators',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    'License :: OSI Approved :: MIT License',
]

DEPENDENCIES = [
    'antlr4-python3-runtime~=4.7.2',
    'azure-appconfiguration~=1.1.1',
    'azure-batch~=11.0.0',
    'azure-cli-core=={}'.format(VERSION),
    'azure-cosmos~=3.0,>=3.0.2',
    'azure-datalake-store~=0.0.49',
    'azure-graphrbac~=0.60.0',
    'azure-identity',
    'azure-keyvault-administration==4.0.0b3',
    'azure-keyvault-keys~=4.5.0b4',
    'azure-keyvault~=1.1.0',
    'azure-loganalytics~=0.1.0',
    'azure-mgmt-advisor==9.0.0',
    'azure-mgmt-apimanagement~=0.2.0',
    'azure-mgmt-appconfiguration~=2.0.0',
    'azure-mgmt-applicationinsights~=1.0.0',
    'azure-mgmt-authorization~=0.61.0',
    'azure-mgmt-batchai~=7.0.0b1',
    'azure-mgmt-batch~=16.0.0',
    'azure-mgmt-billing==6.0.0',
    'azure-mgmt-botservice~=0.3.0',
    'azure-mgmt-cdn==11.0.0',
    'azure-mgmt-cognitiveservices~=12.0.0',
    'azure-mgmt-compute~=23.1.0',
    'azure-mgmt-consumption~=2.0',
    'azure-mgmt-containerinstance~=9.1.0',
    'azure-mgmt-containerregistry==8.2.0',
    'azure-mgmt-containerservice~=16.1.0',
    'azure-mgmt-cosmosdb~=7.0.0b2',
    'azure-mgmt-databoxedge~=1.0.0',
    'azure-mgmt-datalake-analytics~=0.2.1',
    'azure-mgmt-datalake-store~=0.5.0',
    'azure-mgmt-datamigration~=10.0.0',
    'azure-mgmt-deploymentmanager~=0.2.0',
    'azure-mgmt-devtestlabs~=4.0',
    'azure-mgmt-dns~=8.0.0',
    'azure-mgmt-eventgrid==9.0.0',
    'azure-mgmt-eventhub~=9.1.0',
    'azure-mgmt-extendedlocation~=1.0.0b2',
    'azure-mgmt-hdinsight~=9.0.0',
    'azure-mgmt-imagebuilder~=0.4.0',
    'azure-mgmt-iotcentral~=9.0.0b1',
    'azure-mgmt-iothub==2.1.0',
    'azure-mgmt-iothubprovisioningservices~=0.3.0',
    'azure-mgmt-keyvault==9.3.0',
    'azure-mgmt-kusto~=0.3.0',
    'azure-mgmt-loganalytics~=11.0.0',
    'azure-mgmt-managedservices~=1.0',
    'azure-mgmt-managementgroups~=0.1',
    'azure-mgmt-maps~=2.0.0',
    'azure-mgmt-marketplaceordering==1.1.0',
    'azure-mgmt-media~=7.0',
    'azure-mgmt-monitor~=2.0.0',
    'azure-mgmt-msi~=0.2',
    'azure-mgmt-netapp~=5.1.0',
    'azure-mgmt-network~=19.3.0',
    'azure-mgmt-policyinsights~=1.0.0',
    'azure-mgmt-privatedns~=1.0.0',
    'azure-mgmt-rdbms~=10.0.0',
    'azure-mgmt-recoveryservicesbackup~=3.0.0',
    'azure-mgmt-recoveryservices~=2.0.0',
    'azure-mgmt-redhatopenshift==1.0.0',
    'azure-mgmt-redis~=13.0.0',
    'azure-mgmt-relay~=0.1.0',
    'azure-mgmt-reservations==0.6.0',  # TODO: Use requirements.txt instead of '==' #9781
    'azure-mgmt-resource==19.0.0',
    'azure-mgmt-search~=8.0',
    'azure-mgmt-security~=2.0.0b1',
    'azure-mgmt-servicebus~=6.0.0',
    'azure-mgmt-servicefabricmanagedclusters~=1.0.0',
    'azure-mgmt-servicelinker==1.0.0b1',
    'azure-mgmt-servicefabric~=1.0.0',
    'azure-mgmt-signalr~=1.0.0b2',
    'azure-mgmt-sqlvirtualmachine~=1.0.0b1',
    'azure-mgmt-sql~=3.0.1',
    'azure-mgmt-storage~=19.0.0',
    'azure-mgmt-synapse~=2.1.0b2',
    'azure-mgmt-trafficmanager~=0.51.0',
    'azure-mgmt-web~=4.0.0',
    'azure-multiapi-storage~=0.7.0',
    'azure-storage-common~=1.4',
    'azure-synapse-accesscontrol~=0.5.0',
    'azure-synapse-artifacts~=0.8.0',
    'azure-synapse-managedprivateendpoints~=0.3.0',
    'azure-synapse-spark~=0.2.0',
    'chardet~=3.0.4',
    'colorama~=0.4.4',
    'fabric~=2.4',
    'javaproperties~=0.5.1',
    'jsondiff~=1.3.0',
    'packaging~=20.9',
    'PyGithub~=1.38',
    'PyNaCl~=1.4.0',
    'scp~=0.13.2',
    'semver==2.13.0',
    'six>=1.10.0',  # six is still used by countless extensions
    'sshtunnel~=0.1.4',
    'websocket-client~=0.56.0',
    'xmltodict~=0.12'
]

# On Linux, the distribution (Ubuntu, Debian, etc) and version are checked
if sys.platform == 'linux':
    DEPENDENCIES.append('distro')

with open('README.rst', 'r', encoding='utf-8') as f:
    README = f.read()

setup(
    name='azure-cli',
    version=VERSION,
    description='Microsoft Azure Command-Line Tools',
    long_description=README,
    license='MIT',
    author='Microsoft Corporation',
    author_email='azpycli@microsoft.com',
    url='https://github.com/Azure/azure-cli',
    zip_safe=False,
    classifiers=CLASSIFIERS,
    scripts=[
        'az',
        'az.completion.sh',
        'az.bat',
    ],
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests", "azure", "azure.cli"]),
    install_requires=DEPENDENCIES,
    python_requires='>=3.6.0',
    package_data={
        'azure.cli.command_modules.acr': ['*.json'],
        'azure.cli.command_modules.botservice': ['*.json', '*.config'],
        'azure.cli.command_modules.monitor.operations': ['autoscale-parameters-template.json'],
        'azure.cli.command_modules.servicefabric': [
            'template/windows/template.json',
            'template/windows/parameter.json',
            'template/linux/template.json',
            'template/linux/parameter.json',
            'template/service/template.json',
            'template/service/parameter.json'
        ],
        'azure.cli.command_modules.appservice': [
            'resources/WindowsFunctionsStacks.json',
            'resources/LinuxFunctionsStacks.json',
            'resources/WebappRuntimeStacks.json',
            'resources/GenerateRandomAppNames.json'
        ],
        'azure.cli.command_modules.rdbms': [
            'randomname/adjectives.txt',
            'randomname/nouns.txt',
            'templates/mysql_githubaction_template.yaml',
            'templates/postgresql_githubaction_template.yaml'
        ]
    },
    cmdclass=cmdclass
)
